@@ -41,9 +41,6 @@
 struct NautilusTrashFileDetails {
 	NautilusTrashDirectory *trash_directory;
 
-	guint add_directory_connection_id;
-	guint remove_directory_connection_id;
-
 	GList *files;
 
 	GHashTable *callbacks;
@@ -55,8 +52,8 @@ typedef struct {
 	NautilusFileCallback callback;
 	gpointer callback_data;
 
-	GList *delegated_attributes;
-	GList *non_delegated_attributes;
+	NautilusFileAttributes delegated_attributes;
+	NautilusFileAttributes non_delegated_attributes;
 
 	GList *non_ready_files;
 
@@ -66,62 +63,43 @@ typedef struct {
 typedef struct {
 	NautilusTrashFile *trash;
 
-	GList *delegated_attributes;
-	GList *non_delegated_attributes;
+	NautilusFileAttributes delegated_attributes;
+	NautilusFileAttributes non_delegated_attributes;
 } TrashMonitor;
 
-static const char * const delegated_attributes[] = {
-	NAUTILUS_FILE_ATTRIBUTE_DEEP_COUNTS,
-	NAUTILUS_FILE_ATTRIBUTE_DIRECTORY_ITEM_COUNT,
-	NAUTILUS_FILE_ATTRIBUTE_DIRECTORY_ITEM_MIME_TYPES
-};
-
 static void nautilus_trash_file_init       (gpointer   object,
 						  gpointer   klass);
 static void nautilus_trash_file_class_init (gpointer   klass);
 
 EEL_CLASS_BOILERPLATE (NautilusTrashFile,
-				   nautilus_trash_file,
-				   NAUTILUS_TYPE_FILE)
+		       nautilus_trash_file,
+		       NAUTILUS_TYPE_FILE)
 
-static gboolean
-is_delegated_attribute (const char *attribute)
+static NautilusFileAttributes 
+get_delegated_attributes_mask (void)
 {
-	guint i;
-
-	g_return_val_if_fail (attribute != NULL, FALSE);
-
-	for (i = 0; i < G_N_ELEMENTS (delegated_attributes); i++) {
-		if (strcmp (attribute, delegated_attributes[i]) == 0) {
-			return TRUE;
-		}
-	}
-	return FALSE;
-}
-
-static gboolean
-is_delegated_attribute_predicate (gpointer data,
-				  gpointer callback_data)
-{
-	return is_delegated_attribute (data);
+	return NAUTILUS_FILE_ATTRIBUTE_DEEP_COUNTS |
+		NAUTILUS_FILE_ATTRIBUTE_DIRECTORY_ITEM_COUNT |
+		NAUTILUS_FILE_ATTRIBUTE_DIRECTORY_ITEM_MIME_TYPES;
 }
 
 static void
-partition_attributes (GList *attributes,
-		      GList **delegated_attributes,
-		      GList **non_delegated_attributes)
+partition_attributes (NautilusFileAttributes attributes,
+		      NautilusFileAttributes *delegated_attributes,
+		      NautilusFileAttributes *non_delegated_attributes)
 {
-	*delegated_attributes = eel_g_list_partition
-		(attributes,
-		 is_delegated_attribute_predicate,
-		 NULL,
-		 non_delegated_attributes);
+	NautilusFileAttributes mask;
+
+	mask = get_delegated_attributes_mask ();
+
+	*delegated_attributes = attributes & mask;
+	*non_delegated_attributes = attributes & ~mask;
 }
 
 static void             
 real_monitor_add (NautilusFile *file,
 		  gconstpointer client,
-		  GList *attributes)
+		  NautilusFileAttributes attributes)
 {
 	nautilus_directory_monitor_add_internal
 		(file->details->directory, file,
@@ -138,7 +116,7 @@ real_monitor_remove (NautilusFile *file,
 
 static void
 real_call_when_ready (NautilusFile *file,
-		      GList *attributes,
+		      NautilusFileAttributes attributes,
 		      NautilusFileCallback callback,
 		      gpointer callback_data)
 
@@ -160,7 +138,7 @@ real_cancel_call_when_ready (NautilusFile *file,
 
 static gboolean
 real_check_if_ready (NautilusFile *file,
-		     GList *attributes)
+		     NautilusFileAttributes attributes)
 {
 	return nautilus_directory_check_if_ready_internal
 		(file->details->directory, file,
@@ -197,8 +175,6 @@ trash_callback_destroy (TrashCallback *trash_callback)
 	g_assert (NAUTILUS_IS_TRASH_FILE (trash_callback->trash));
 
 	nautilus_file_unref (NAUTILUS_FILE (trash_callback->trash));
-	eel_g_list_free_deep (trash_callback->delegated_attributes);
-	eel_g_list_free_deep (trash_callback->non_delegated_attributes);
 	g_list_free (trash_callback->non_ready_files);
 	g_free (trash_callback);
 }
@@ -284,10 +260,8 @@ add_real_file (NautilusTrashFile *trash,
 	trash->details->files = g_list_prepend
 		(trash->details->files, real_file);
 	
-	g_signal_connect (real_file,
-			    "changed",
-			    G_CALLBACK (real_file_changed_callback),
-			    trash);
+	g_signal_connect_object (real_file, "changed",
+				 G_CALLBACK (real_file_changed_callback), trash, 0);
 
 	/* Add the file to any extant monitors. */
 	g_hash_table_foreach (trash->details->monitors,
@@ -297,8 +271,8 @@ add_real_file (NautilusTrashFile *trash,
 
 static void
 trash_callback_remove_file_cover (gpointer key,
-				       gpointer value,
-				       gpointer callback_data)
+				  gpointer value,
+				  gpointer callback_data)
 {
 	trash_callback_remove_file
 		(value, NAUTILUS_FILE (callback_data));
@@ -330,9 +304,8 @@ remove_real_file (NautilusTrashFile *trash,
 		 monitor_remove_file,
 		 real_file);
 
-	gtk_signal_disconnect_by_func (GTK_OBJECT (real_file),
-				       G_CALLBACK (real_file_changed_callback),
-				       trash);
+	g_signal_handlers_disconnect_by_func
+		(real_file, G_CALLBACK (real_file_changed_callback), trash);
 
 	trash->details->files = g_list_remove
 		(trash->details->files, real_file);
@@ -388,7 +361,7 @@ remove_directory_callback (NautilusTrashDirectory *trash_directory,
 
 static void
 trash_file_call_when_ready (NautilusFile *file,
-			    GList *attributes,
+			    NautilusFileAttributes attributes,
 			    NautilusFileCallback callback,
 			    gpointer callback_data)
 
@@ -415,7 +388,7 @@ trash_file_call_when_ready (NautilusFile *file,
 	trash_callback->callback_data = callback_data;
 	trash_callback->initializing = TRUE;
 
-	partition_attributes (eel_g_str_list_copy (attributes),
+	partition_attributes (attributes,
 			      &trash_callback->delegated_attributes,
 			      &trash_callback->non_delegated_attributes);
 
@@ -482,16 +455,16 @@ trash_file_cancel_call_when_ready (NautilusFile *file,
 
 static gboolean
 trash_file_check_if_ready (NautilusFile *file,
-			   GList *attributes)
+			   NautilusFileAttributes attributes)
 {
-	GList *delegated_attributes, *non_delegated_attributes;
+	NautilusFileAttributes delegated_attributes, non_delegated_attributes;
 	NautilusTrashFile *trash;
 	GList *node;
 	gboolean ready;
 
 	trash = NAUTILUS_TRASH_FILE (file);
 
-	partition_attributes (g_list_copy (attributes),
+	partition_attributes (attributes,
 			      &delegated_attributes,
 			      &non_delegated_attributes);
 
@@ -507,16 +480,13 @@ trash_file_check_if_ready (NautilusFile *file,
 		}
 	}
 
-	g_list_free (delegated_attributes);
-	g_list_free (non_delegated_attributes);
-
 	return ready;
 }
 
 static void
 trash_file_monitor_add (NautilusFile *file,
 			gconstpointer client,
-			GList *attributes)
+			NautilusFileAttributes attributes)
 {
 	NautilusTrashFile *trash;
 	TrashMonitor *monitor;
@@ -530,8 +500,6 @@ trash_file_monitor_add (NautilusFile *file,
 	monitor = g_hash_table_lookup (trash->details->monitors, client);
 	if (monitor != NULL) {
 		g_assert (monitor->trash == trash);
-		eel_g_list_free_deep (monitor->delegated_attributes);
-		eel_g_list_free_deep (monitor->non_delegated_attributes);
 	} else {
 		monitor = g_new0 (TrashMonitor, 1);
 		monitor->trash = trash;
@@ -539,7 +507,7 @@ trash_file_monitor_add (NautilusFile *file,
 				     (gpointer) client, monitor);
 	}
 
-	partition_attributes (eel_g_str_list_copy (attributes),
+	partition_attributes (attributes,
 			      &monitor->delegated_attributes,
 			      &monitor->non_delegated_attributes);
 
@@ -574,8 +542,6 @@ trash_file_monitor_remove (NautilusFile *file,
 		nautilus_file_monitor_remove (node->data, monitor);
 	}
 
-	eel_g_list_free_deep (monitor->delegated_attributes);
-	eel_g_list_free_deep (monitor->non_delegated_attributes);
 	g_free (monitor);
 }
 
@@ -751,16 +717,10 @@ nautilus_trash_file_init (gpointer object, gpointer klass)
 		(trash_callback_hash, trash_callback_equal);
 	trash_file->details->monitors = g_hash_table_new (NULL, NULL);
 
-	trash_file->details->add_directory_connection_id = gtk_signal_connect
-		(GTK_OBJECT (trash_directory),
-		 "add_real_directory",
-		 G_CALLBACK (add_directory_callback),
-		 trash_file);
-	trash_file->details->remove_directory_connection_id = gtk_signal_connect
-		(GTK_OBJECT (trash_directory),
-		 "remove_real_directory",
-		 G_CALLBACK (remove_directory_callback),
-		 trash_file);
+	g_signal_connect_object (trash_directory, "add_real_directory",
+				 G_CALLBACK (add_directory_callback), trash_file, 0);
+	g_signal_connect_object (trash_directory, "remove_real_directory",
+				 G_CALLBACK (remove_directory_callback), trash_file, 0);
 
 	real_directories = nautilus_merged_directory_get_real_directories
 		(NAUTILUS_MERGED_DIRECTORY (trash_directory));
@@ -771,7 +731,7 @@ nautilus_trash_file_init (gpointer object, gpointer klass)
 }
 
 static void
-trash_destroy (GtkObject *object)
+trash_finalize (GObject *object)
 {
 	NautilusTrashFile *trash_file;
 	NautilusTrashDirectory *trash_directory;
@@ -788,11 +748,6 @@ trash_destroy (GtkObject *object)
 		g_warning ("file monitor still active when trash virtual file is destroyed");
 	}
 
-	gtk_signal_disconnect (GTK_OBJECT (trash_directory),
-			       trash_file->details->add_directory_connection_id);
-	gtk_signal_disconnect (GTK_OBJECT (trash_directory),
-			       trash_file->details->remove_directory_connection_id);
-
 	g_hash_table_destroy (trash_file->details->callbacks);
 	g_hash_table_destroy (trash_file->details->monitors);
 
@@ -800,19 +755,19 @@ trash_destroy (GtkObject *object)
 
 	nautilus_directory_unref (NAUTILUS_DIRECTORY (trash_directory));
 
-	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
 static void
 nautilus_trash_file_class_init (gpointer klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class;
 	NautilusFileClass *file_class;
 
-	object_class = GTK_OBJECT_CLASS (klass);
+	object_class = G_OBJECT_CLASS (klass);
 	file_class = NAUTILUS_FILE_CLASS (klass);
 	
-	object_class->destroy = trash_destroy;
+	object_class->finalize = trash_finalize;
 
 	file_class->monitor_add = trash_file_monitor_add;
 	file_class->monitor_remove = trash_file_monitor_remove;
